@@ -72,7 +72,6 @@ import Data.Iteratee.Base
 import Control.Exception
 import Control.Monad.Trans.Class
 import Data.Maybe
-import Data.Monoid
 import Data.Typeable
 
 -- exception helpers
@@ -246,6 +245,7 @@ eneeCheckIfDoneHandle h f inner = Iteratee $ \od oc ->
       onCont k Nothing  = runIter (f k Nothing) od oc
       onCont k (Just e) = runIter (h k e)       od oc
   in runIter inner onDone onCont
+{-# INLINABLE eneeCheckIfDoneHandle #-}
 
 eneeCheckIfDonePass
   :: (Monad m, NullPoint elo)
@@ -255,7 +255,7 @@ eneeCheckIfDonePass
      )
   -> Enumeratee elo eli m a
 eneeCheckIfDonePass f = eneeCheckIfDoneHandle (\k e -> f k (Just e)) f
-{-# INLINEABLE eneeCheckIfDonePass #-}
+{-# INLINABLE eneeCheckIfDonePass #-}
 
 eneeCheckIfDoneIgnore
   :: (Monad m, NullPoint elo)
@@ -293,6 +293,7 @@ mapChunksM f = eneeCheckIfDonePass (icont . step)
 {-# INLINE mapChunksM #-}
 
 -- |Convert one stream into another, not necessarily in lockstep.
+-- 
 -- The transformer mapStream maps one element of the outer stream
 -- to one element of the nested stream.  The transformer below is more
 -- general: it may take several elements of the outer stream to produce
@@ -308,6 +309,7 @@ convStream fi = eneeCheckIfDonePass check
     check k (Just e) = throwRecoverableErr e (const identity) >> check k Nothing
     check k _ = isStreamFinished >>= maybe (step k) (idone (liftI k) . EOF . Just)
     step k = fi >>= eneeCheckIfDonePass check . k . Chunk
+{-# INLINABLE convStream #-}
 
 -- |The most general stream converter.  Given a function to produce iteratee
 -- transformers and an initial state, convert the stream using iteratees
@@ -326,7 +328,7 @@ unfoldConvStream f acc0 = eneeCheckIfDonePass (check acc0)
                     eneeCheckIfDonePass (check acc') . k . Chunk $ s'
 
 unfoldConvStreamCheck
-  :: (Monad m, Nullable elo, Monoid elo)
+  :: (Monad m, Nullable elo)
   => (((Stream eli -> Iteratee eli m a)
         -> Maybe SomeException
         -> Iteratee elo m (Iteratee eli m a)
@@ -346,6 +348,7 @@ unfoldConvStreamCheck checkDone f acc0 = checkDone (check acc0)
       let i = f acc >>= \(acc', s') ->
                            (checkDone (check acc') . k $ Chunk s')
       in joinIM $ enumChunk str' i
+{-# INLINABLE unfoldConvStreamCheck #-}
 
 -- | Collapse a nested iteratee.  The inner iteratee is terminated by @EOF@.
 --   Errors are propagated through the result.
@@ -374,7 +377,8 @@ joinIM mIter = Iteratee $ \od oc -> mIter >>= \iter -> runIter iter od oc
 
 -- ------------------------------------------------------------------------
 -- Enumerators
--- |Each enumerator takes an iteratee and returns an iteratee
+-- | Each enumerator takes an iteratee and returns an iteratee
+-- 
 -- an Enumerator is an iteratee transformer.
 -- The enumerator normally stops when the stream is terminated
 -- or when the iteratee moves to the done state, whichever comes first.
@@ -415,6 +419,7 @@ enumErr e iter = runIter iter onDone onCont
 
 
 -- |The composition of two enumerators: essentially the functional composition
+-- 
 -- It is convenient to flip the order of the arguments of the composition
 -- though: in e1 >>> e2, e1 is executed first
 
