@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleContexts, BangPatterns, TupleSections #-}
+{-# LANGUAGE FlexibleContexts, BangPatterns, TupleSections, ScopedTypeVariables #-}
 
 -- |Monadic Iteratees:
 -- incremental input parsers, processors and transformers
@@ -58,6 +58,8 @@ module Data.Iteratee.ListLike (
   ,zip4
   ,zip5
   ,sequence_
+  ,countConsumed
+  ,greedy
   -- ** Monadic functions
   ,mapM_
   ,foldM
@@ -70,17 +72,16 @@ import Prelude hiding (mapM_, null, head, last, drop, dropWhile, take, takeWhile
 
 import qualified Prelude as Prelude
 
+import Data.List (partition)
 import qualified Data.ListLike as LL
 import qualified Data.ListLike.FoldableLL as FLL
 import Data.Iteratee.Iteratee
 import Data.Monoid
-import Data.Maybe (catMaybes)
 import Control.Applicative ((<$>), (<*>), (<*))
 import Control.Monad (liftM, liftM2, mplus, (<=<))
 import Control.Monad.Trans.Class
 import Data.Word (Word8)
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Char8 as BC
 
 -- Useful combinators for implementing iteratees and enumerators
 
@@ -354,9 +355,7 @@ take n' iter
       | otherwise          = idone (k (Chunk s1)) (Chunk s2)
       where (s1, s2) = LL.splitAt n str
     step _n k stream       = idone (k stream) stream
-{-# SPECIALIZE take :: Monad m => Int -> Enumeratee [el] [el] m a #-}
-{-# SPECIALIZE take :: Monad m => Int -> Enumeratee B.ByteString B.ByteString m a #-}
-{-# SPECIALIZE take :: Monad m => Int -> Enumeratee BC.ByteString BC.ByteString m a #-}
+{-# INLINE take #-}
 
 -- |Read n elements from a stream and apply the given iteratee to the
 -- stream of the read elements. If the given iteratee accepted fewer
@@ -410,9 +409,7 @@ takeUpTo i iter
                 Left  (a,s')       -> od' (idone a s') (Chunk s2)
                 Right (k',e)       -> od' (icont k' e) (Chunk s2)
     step _ k stream       = idone (k stream) stream
-{-# SPECIALIZE takeUpTo :: Monad m => Int -> Enumeratee [el] [el] m a #-}
-{-# SPECIALIZE takeUpTo :: Monad m => Int -> Enumeratee B.ByteString B.ByteString m a #-}
-{-# INLINABLE takeUpTo #-}
+{-# INLINE takeUpTo #-}
 
 -- | Takes an element predicate and returns the (possibly empty)
 -- prefix of the stream. All characters
@@ -930,20 +927,16 @@ sequence_ = self
           -- give a chunk to each iteratee
           is'  <- lift $ mapM (enumChunk s) is
           -- filter done iteratees
-          is'' <- lift $ catMaybes `liftM` mapM checkIfDone is'
-          if Prelude.null is''
-            then idone () <=< remainingStream $ is'
-            else self is''
+          (done, notDone) <- lift $ partition fst `liftM` mapM enumCheckIfDone is'
+          if Prelude.null notDone
+            then idone () <=< remainingStream $ map snd done
+            else self $ map snd notDone
         step s@(EOF _) = do
           s' <- remainingStream <=< lift $ mapM (enumChunk s) is
           case s' of
             EOF (Just e) -> throwErr e
             _            -> idone () s'
 
-        checkIfDone i = runIter i
-            (\_ _ -> return Nothing)
-            (\k e -> return $ Just $ icont k e)
-
     -- returns the unconsumed part of the stream; "sequence_ is" consumes as
     -- much of the stream as the iteratee in is that consumes the most; e.g.
     -- sequence_ [I.head, I.last] consumes whole stream
@@ -967,6 +960,28 @@ sequence_ = self
     shorter e@(EOF _) _         = e
     shorter _         e@(EOF _) = e
 
+-- |Transform an iteratee into one that keeps track of how much data it
+-- consumes.
+countConsumed :: forall a s el m n.
+                 (Monad m, LL.ListLike s el, Nullable s, Integral n) =>
+                 Iteratee s m a
+              -> Iteratee s m (a, n)
+countConsumed i = go 0 (const i) (Chunk empty)
+  where
+    go :: n -> (Stream s -> Iteratee s m a) -> Stream s
+       -> Iteratee s m (a, n)
+    go !n f str@(EOF _) = (, n) `liftM` f str
+    go !n f str@(Chunk c) = Iteratee rI
+      where
+        newLen = n + fromIntegral (LL.length c)
+        rI od oc = runIter (f str) onDone onCont
+          where
+            onDone a str'@(Chunk c') =
+                od (a, newLen - fromIntegral (LL.length c')) str'
+            onDone a str'@(EOF _) = od (a, n) str'
+            onCont f' mExc = oc (go newLen f') mExc
+{-# INLINE countConsumed #-}
+
 -- ------------------------------------------------------------------------
 -- Enumerators
 
@@ -986,6 +1001,49 @@ enumPureNChunk str n iter
                        in runIter iter' idoneM on_cont
 {-# INLINE enumPureNChunk #-}
 
+-- | Convert an iteratee to a \"greedy\" version.
+--
+-- When a chunk is received, repeatedly run the input iteratee
+-- until the entire chunk is consumed, then the outputs
+-- are combined (via 'mconcat').
+--
+-- > > let l = [1..5::Int]
+-- > > run =<< enumPure1Chunk l (joinI (take 2 stream2list))
+-- > [1,2]
+-- > > run =<< enumPure1Chunk l (greedy $ joinI (I.take 2 stream2list))
+-- > [1,2,3,4,5]
+--
+-- Note that a greedy iteratee will consume the entire input chunk and force
+-- the next chunk before returning a value.  A portion of the second chunk may
+-- be consumed.
+-- 
+-- 'greedy' may be useful on the first parameter of 'convStream', e.g.
+-- 
+-- > convStream (greedy someIter)
+--
+-- to create more efficient converters.
+greedy ::
+ (Monad m, Functor m, LL.ListLike s el', Monoid a) =>
+  Iteratee s m a
+  -> Iteratee s m a
+greedy iter' = liftI (step [] iter')
+ where
+  step acc iter (Chunk str)
+    | LL.null str = liftI (step acc iter)
+    | otherwise   = joinIM $ do
+      i2 <- enumPure1Chunk str iter
+      result <- runIter i2 (\a s -> return $ Left (a,s))
+                           (\k e -> return $ Right (icont k e))
+      case result of
+        Left (a, Chunk resS)
+          | LL.null resS
+              || LL.length resS == LL.length str -> return $
+                         idone (mconcat $ reverse (a:acc)) (Chunk resS)
+        Left (a, stream) -> return $ step (a:acc) iter stream
+        Right i -> return $ fmap (mconcat . reverse . (:acc)) i
+  step acc iter stream = joinIM $
+    enumChunk stream (fmap (mconcat . reverse . (:acc)) iter)
+{-# INLINE greedy #-}
 
 -- ------------------------------------------------------------------------
 -- Monadic functions
