@@ -1,5 +1,5 @@
 {-# OPTIONS_GHC -O #-}
-{-# LANGUAGE NoMonomorphismRestriction #-}
+{-# LANGUAGE NoMonomorphismRestriction, ViewPatterns #-}
 
 import Prelude as P
 
@@ -7,7 +7,9 @@ import QCUtils
 
 import Test.Framework (defaultMain, testGroup)
 import Test.Framework.Providers.QuickCheck2 (testProperty)
+import Test.Framework.Providers.HUnit (testCase)
 
+import Test.HUnit
 import Test.QuickCheck
 
 import           Data.Iteratee hiding (head, break)
@@ -240,6 +242,10 @@ prop_enumWith3 xs i n =
    ==  runner1 (enumSpecial xs n (i >> stream2list))
  where types = (xs :: [Int], i :: I)
 
+prop_countConsumed (Positive (min (2^10) -> n)) (Positive (min (2^20) -> a)) (Positive k) =
+            runner1 (enumPureNChunk [1..] n iter) == (a, a)
+  where
+    iter = countConsumed . joinI $ (takeUpTo (a + k) ><> Iter.take a) Iter.last
 
 -- ---------------------------------------------
 -- Nested Iteratees
@@ -410,6 +416,21 @@ prop_zip xs i1 i2 n = n > 0 ==>
      in ((r1,r2), shorter)
  where types = (i1 :: I, i2 :: I, xs :: [Int])
 
+-- ---------------------------------------------
+-- Sequences
+
+test_sequence_ =
+  assertEqual "sequence_: no duplicate runs" ((),[4,5])
+              (runWriter (Iter.enumList [[4],[5::Int]] (Iter.sequence_ [iter])
+                          >>= run))
+ where
+  iter = do
+    x <- Iter.head
+    lift $ tell [x]
+    y <- Iter.head
+    lift $ tell [y]
+
+
 -- ---------------------------------------------
 -- Data.Iteratee.Char
 
@@ -498,6 +519,7 @@ tests = [
     testProperty "enumWith" prop_enumWith
     ,testProperty "enumWith remaining" prop_enumWith2
     ,testProperty "enumWith remaining 2" prop_enumWith3
+    ,testProperty "countConsumed" prop_countConsumed
     ]
   ,testGroup "Folds" [
     testProperty "foldl" prop_foldl
@@ -509,6 +531,7 @@ tests = [
    ]
   ,testGroup "Zips" [
     testProperty "zip" prop_zip
+   ,testCase     "sequence_" test_sequence_
    ]
   ,testGroup "Data.Iteratee.Char" [
     --testProperty "line" prop_line
